/* GLapplic.cpp
*
*/
#define WIN32_LEAN_AND_MEAN
#include <stdlib.h>
#include <windows.h>
#include "GLapplic.h"

void SetDCPixelFormat (HDC hdc);

class Module
{
public:
	static HINSTANCE hInstance;
	static PSTR szCmdLine;
	static int iCmdShow;
	static PSTR Name;
	static void RegisterWindow();
};

///////////////////////////////////////////////////////////////////////////////
// CAppWindow class

CAppWindow::CAppWindow(PSTR szName)
{
	Module::Name = Name = szName; // classname = Window caption, for simplicity
	hpal = 0;
	hrc = 0;
	hWnd = 0;
	Width = Height = CW_USEDEFAULT;
}

CAppWindow::~CAppWindow()
{
	if (hpal) DeleteObject(hpal);
}

void CAppWindow::Create()
{
	Module::RegisterWindow();
	/*
	* Pass 'this' pointer in lpParam of CreateWindow()
	* The first time WM_CREATE is called, WndProc will
	* store it in the "extra" bytes reserved for each instance
	* of the program (see RegisterWindow). WndProc uses this information
	* to call the MsgProc member function of the object.
	*/
	hWnd = CreateWindow(
		Module::Name, // window class name
		Name,         // window caption
		WS_OVERLAPPEDWINDOW, // window style
						     // | WS_VSCROLL if vertical scroll needed
		CW_USEDEFAULT,  // initial x position;
		CW_USEDEFAULT,  // initial y position;
		Width + 2*GetSystemMetrics(SM_CXFRAME),   // inital x size
		Height+ 2*GetSystemMetrics(SM_CYFRAME)
			    + GetSystemMetrics(SM_CYCAPTION), // initial y size
		NULL,   // parent window handle
		NULL,   // window menu handle
		Module::hInstance, // program instance handle
		(PSTR) this // creation parameters
	);
	if (!hWnd) exit(-1);
	// Create a rendering context.
	HDC hdc = GetDC(hWnd);
	SetDCPixelFormat(hdc);
	hrc = wglCreateContext(hdc);
	// Call user initialization
	SetupWindow();
	wglMakeCurrent(hdc,hrc);
	PrepareScene();
	wglMakeCurrent(NULL,NULL);
	ReleaseDC(hWnd,hdc);
 	ShowWindow(hWnd, Module::iCmdShow);
	UpdateWindow(hWnd);
}

int CAppWindow::Run()
{
	MSG eventMessage;
	Create();
	// Keep running until the user kills the program
	while (GetMessage(&eventMessage, 0, 0, 0)) {
		TranslateMessage(&eventMessage);
		DispatchMessage(&eventMessage);
	}
	return (int) eventMessage.wParam;
}

void CAppWindow::Destroy()
{
	wglDeleteContext(hrc);
	ShowWindow(hWnd, SW_HIDE);
	CleanupWindow();
	PostQuitMessage(0);
}


void CAppWindow::Invalidate(BOOL Flag)
{
	InvalidateRect(hWnd, 0, Flag);
}




// Redefine the viewing volume and viewport when the window size changes.

void CAppWindow::OnSize(int cx, int cy)
{
	GLdouble gldAspect;
	GLsizei glnWidth, glnHeight;

	glnWidth = (GLsizei) cx;
	glnHeight = (GLsizei) cy;
	gldAspect = (GLdouble) glnWidth / (GLdouble) glnHeight;

	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	gluPerspective (30.0,           // Field-of-view angle
					gldAspect,      // Aspect ratio of viewing volume
					1.0,            // Distance to near clipping plane
					100.0);          // Distance to far clipping plane

	glViewport (0, 0, glnWidth, glnHeight);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

	
LRESULT CALLBACK CAppWindow::MsgProc(UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hdc;
	POINT pt;
	switch (iMessage) {
		case WM_SIZE:
			hdc = GetDC (hWnd);
			wglMakeCurrent (hdc, hrc);
			OnSize(LOWORD(lParam),HIWORD(lParam));
			wglMakeCurrent (NULL, NULL);
			ReleaseDC (hWnd, hdc);
			break;
		case WM_SETFOCUS:
			WmSetFocus();
			break;
		case WM_KILLFOCUS:
			WmKillFocus();
			break;
		case WM_ERASEBKGND:
			break;
		case WM_PAINT:
			hdc = BeginPaint(hWnd, &ps);
			wglMakeCurrent(hdc,hrc);
			DrawScene();
			SwapBuffers (hdc);
			wglMakeCurrent(NULL,NULL);
			EndPaint(hWnd, &ps);
			break;
		case WM_CHAR:
			WmChar(wParam, lParam);
			break;
		case WM_COMMAND:
			WmCommand(wParam, lParam);
			break;
		case WM_DRAWITEM:
			WmDrawItem(wParam, lParam);
			break;
		case WM_KEYDOWN:
			WmKeyDown(wParam, lParam);
			break;
		case WM_LBUTTONDOWN:
			pt.x = LOWORD(lParam);
			pt.y = HIWORD(lParam);
			WmLButtonDown((int) wParam, pt);
			break;
		case WM_LBUTTONUP:
			pt.x = LOWORD(lParam);
			pt.y = HIWORD(lParam);
			WmLButtonUp((int) wParam, pt);
			break;
		case WM_MOUSEMOVE:
			pt.x = LOWORD(lParam);
			pt.y = HIWORD(lParam);
			WmMouseMove((int) wParam, pt);
			break;
		case WM_TIMER:
			WmTimer();
			break;
		case WM_VSCROLL:
			WmVScroll(wParam,lParam);
			break;
		case WM_QUERYNEWPALETTE:
			SetPalette(FALSE); // Realize in the foreground
			break;
		case WM_PALETTECHANGED:
			// See if the change was caused by us and ignore it if not.
			if ( (HWND) wParam != hWnd) {
				SetPalette(TRUE); // Realize in the background. 
			}
			break;
		case WM_DESTROY:
			Destroy();
			break;
		default:
			return DefWindowProc(hWnd, iMessage, wParam, lParam);
	}
	return 0;
}

// helper routine to keep palette updated

int CAppWindow::SetPalette(BOOL bForceBkgnd)
{
	if (!hpal) return 0;
	// We are going active, so realize our palette.
	HDC hdc = GetDC(hWnd);
	HPALETTE oldpalette = SelectPalette(hdc,hpal,bForceBkgnd);
	int NumColorsChanged = RealizePalette(hdc);
	SelectPalette(hdc, oldpalette, TRUE);
	RealizePalette(hdc);
	ReleaseDC(hWnd,hdc);
	// If any colors have changed or we are in the
	// background, repaint the lot.
	if (NumColorsChanged || bForceBkgnd) {
		InvalidateRect(hWnd, NULL, TRUE); // Repaint.
	}
	return NumColorsChanged;
}

// helper routines to access extra bytes in window

inline CAppWindow *GetPointer(HWND hWnd)
{
	return (CAppWindow *) GetWindowLongPtr(hWnd, 0);
}

inline void SetPointer(HWND hWnd, CAppWindow *pWindow)
{
	SetWindowLongPtr(hWnd, 0, (LONG_PTR) pWindow);
}

// Routines to access static Module data

HINSTANCE GetInstance()
{
	return Module::hInstance;
}

PSTR GetCmdLine()
{
	return Module::szCmdLine;
}

/////////////////////////////////////////////////////////////////////////
// Module class implementation


HINSTANCE Module::hInstance;
PSTR Module::szCmdLine;
int Module::iCmdShow;
PSTR Module::Name;

LRESULT CALLBACK WndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam);

void Module::RegisterWindow()
{
	WNDCLASS wc;
	PSTR szRes = "APPLIC";
	HICON hicon = LoadIcon(hInstance,szRes);
	HCURSOR hcursor = LoadCursor(hInstance,szRes);

	//wc.cbSize = sizeof(wndclass);
	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = WndProc;
	wc.cbClsExtra = 0;
	/* Reserve extra bytes for a pointer to the C++
	*  (CappWindow) object corresponding to the window.
	*/
	wc.cbWndExtra = sizeof (CAppWindow *);
	wc.hInstance = hInstance;
	wc.hIcon = (hicon? hicon: LoadIcon(0,IDI_APPLICATION));
	wc.hCursor =  (hcursor? hcursor: LoadCursor(0,IDC_ARROW));
	wc.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
	wc.lpszMenuName = szRes;
	wc.lpszClassName = Name;

	int hAtom = RegisterClass(&wc);
	if (!hAtom) exit(-1);
}

//////////////////////////////////////////////////////////////////////////////////
// Message Handler for this instance

LRESULT CALLBACK WndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	CAppWindow *pWindow = GetPointer(hWnd);
	if (pWindow) {
		return pWindow->MsgProc(iMessage, wParam, lParam);
	}
	else {
		/*
		*  Store a pointer to this object in the window's extra
		*  bytes.  This will enable us to acess this object (and
		*  its member functions) where we are only given
		*  a handle to identify the window.
		*/
		if (iMessage== WM_CREATE) {
			LPCREATESTRUCT lpcs;
			lpcs = (LPCREATESTRUCT) lParam;
			pWindow = (CAppWindow *) lpcs->lpCreateParams;
			SetPointer(hWnd, pWindow);
			return pWindow->MsgProc(iMessage, wParam, lParam);
		}
		else
			return DefWindowProc(hWnd, iMessage, wParam, lParam);
	}
}

/*
 *  SetDCPixelFormat sets the pixel format for a device context in
 *  preparation for creating a rendering context.
 *
 *  Input parameters:
 *    hdc = Device context handle
 *
 *  Returns:
 *    Nothing
 */

void SetDCPixelFormat (HDC hdc)
{
	int flags = PFD_DRAW_TO_WINDOW |
				PFD_SUPPORT_OPENGL |
				PFD_DOUBLEBUFFER;
					  
	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof (PIXELFORMATDESCRIPTOR),             // Size of this structure
		1,                                          // Version number
		flags,                                      // Flags
		PFD_TYPE_RGBA,                              // RGBA pixel values
		24,                                         // 24-bit color
		0, 0, 0, 0, 0, 0,                           // Don't care about these
		0, 0,                                       // No alpha buffer
		0, 0, 0, 0, 0,                              // No accumulation buffer
		32,                                         // 32-bit depth buffer
		0,                                          // No stencil buffer
		0,                                          // No auxiliary buffers
		PFD_MAIN_PLANE,                             // Layer type
		0,                                          // Reserved (must be 0)
		0, 0, 0                                     // No layer masks
	};

	int nPixelFormat;

	nPixelFormat = ChoosePixelFormat (hdc, &pfd);
	SetPixelFormat (hdc, nPixelFormat, &pfd);

	DescribePixelFormat(hdc, nPixelFormat, sizeof(PIXELFORMATDESCRIPTOR),&pfd);
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
				   PSTR szCmdLine, int iCmdShow)
{
	extern int Main();

	Module::hInstance = hInstance;
	Module::szCmdLine = szCmdLine;
	Module::iCmdShow = iCmdShow;
	return Main();
}
