/** Sinc.cpp
*
*   Demonstrate OpenGL functions
*
**/
#define WIN32_LEAN_AND_MEAN
#include <stdlib.h>
#include <math.h>
#include <windows.h>
#include "GLapplic.h"

double sinc(double x)
{
	const double pi = 4.0*atan(1.0);
	x = pi*fabs(x);
	return (x? sin(x)/x: 1.0);
}

class MyApp: public CAppWindow
{
public:
	MyApp();
	void OnSize(int cx, int cy);
	void DrawScene();
};

MyApp::MyApp(): CAppWindow("Sinc")
{
	Width  = 480 ;
	Height = 320 ;
}

void MyApp::OnSize(int cx, int cy)
{
	GLsizei glnWidth, glnHeight;

	glnWidth = (GLsizei) cx;
	glnHeight = (GLsizei) cy;
	glMatrixMode(GL_PROJECTION); 
	glLoadIdentity();
	gluOrtho2D(-4.2, 4.2, -0.4, 1.2);
	glViewport(0,0,glnWidth,glnHeight);	
}

void MyApp::DrawScene()
{
	const int NPTS = 401;
	int i;
	double x, y, A, B;
	glClearColor(1.0,1.0,1.0,0.0);       // set white background color
	glClear(GL_COLOR_BUFFER_BIT);     // clear the screen 
	glColor3f(0.0f, 0.0f, 0.0f);          // set the drawing color
	A = 8.0/(NPTS-1);
	B = -4.0;
	glBegin(GL_LINE_STRIP);
	for (i=0; i<NPTS; i++) {
		x = A*i+B;
		y = sinc(x);
		glVertex2f((float)x,(float)y);
	}
	glEnd();	
	glFlush();		                 // send all output to display 
}

int Main()
{
	return MyApp().Run();
}
